@@ -37,6 +37,7 @@
 (require 'thingatpt)
 (require 'pcase)
 (require 'mailcap)
+(require 'seq)
 
 (ignore-errors
   (require 'helm)
@@ -176,6 +177,16 @@ nil: Prompt delete or not."
   :type 'file
   :group 'calibredb)
 
+(defcustom calibredb-ebook-meta-program
+  (cond
+   ((eq system-type 'darwin)
+    "/Applications/calibre.app/Contents/MacOS/ebook-meta")
+   (t
+    "ebook-meta"))
+  "Executable used to get ebook metadata."
+  :type 'file
+  :group 'calibredb)
+
 (defcustom calibredb-fetch-metadata-source-list '("Google" "Amazon.com")
   "Source alist used to fetch ebook metadata."
   :type 'sexp
@@ -255,6 +266,11 @@ Set negative to keep original length."
 (define-obsolete-variable-alias 'calibredb-format-icons
   'calibredb-format-all-the-icons "calibredb 2.3.2")
 
+(defcustom calibredb-format-nerd-icons nil
+  "Set Non-nil to show file format icons with nerd-icons."
+  :group 'calibredb
+  :type 'boolean)
+
 (defcustom calibredb-format-all-the-icons nil
   "Set Non-nil to show file format icons with all-the-icons."
   :group 'calibredb
@@ -407,15 +423,23 @@ LEFT JOIN b
 ON d.book = b.id
 LEFT JOIN identifiers AS i
 ON d.book = i.book
-GROUP BY d.book
+GROUP BY d.book"
+  "TODO calibre database query statement.")
+
+
+(defvar calibredb-sql-count-string "SELECT COUNT(id) FROM (SELECT * FROM (%s) %s)"
+  "calibre database query statement: count total items.")
+
+
+(defvar calibredb-sql-order-by-string "
 ORDER BY
   (CASE
-    WHEN t.tag LIKE '%favorite%' THEN 1
-    WHEN t.tag LIKE '%highlight%' THEN 2
-    WHEN t.tag LIKE '%archive%' THEN 4
-    ELSE 3
-  END),"
-  "TODO calibre database query statement.")
+   WHEN t.tag LIKE '%favorite%' THEN 1
+   WHEN t.tag LIKE '%highlight%' THEN 2
+   WHEN t.tag LIKE '%archive%' THEN 4
+   ELSE 3
+   END),"
+  "calibre database query statement: order by favorite, highlight, archive.")
 
 (defun calibredb-query-search-string (filter)
   "DEPRECATED Return the where part of SQL based on FILTER."
@@ -550,33 +574,32 @@ tell user something’s wrong."
 (defun calibredb-query-to-alist (query-result)
   "Builds alist out of a full `calibredb-query' query record result.
 Argument QUERY-RESULT is the query result generate by sqlite."
-  (if query-result
-      (let ((spl-query-result (if (and (functionp 'sqlite-available-p) (sqlite-available-p))
-                                  query-result
-                                (split-string (calibredb-chomp query-result) calibredb-sql-separator))))
-        `((:id                     ,(let ((id (nth 0 spl-query-result)))
-                                      (if (stringp id)
-                                          id
-                                        (number-to-string id))))
-          (:author-sort            ,(nth 1 spl-query-result))
-          (:book-dir               ,(nth 2 spl-query-result))
-          (:book-name              ,(nth 3 spl-query-result))
-          (:book-format  ,(downcase (or (nth 4 spl-query-result) "")))
-          (:book-pubdate           ,(nth 5 spl-query-result))
-          (:book-title             ,(nth 6 spl-query-result))
-          (:file-path    ,(concat (file-name-as-directory calibredb-root-dir)
-                                  (file-name-as-directory (nth 2 spl-query-result))
-                                  (nth 3 spl-query-result) "." (downcase (or (nth 4 spl-query-result) ""))))
-          (:tag                    ,(or (nth 7 spl-query-result) ""))
-          (:size                   ,(format "%.2f" (/ (let ((size (or (nth 8 spl-query-result) "")))
-                                                        (if (stringp size)
-                                                            (string-to-number size) size)) 1048576.0)))
-          (:comment                ,(or (nth 9 spl-query-result) ""))
-          (:ids                    ,(or (nth 10 spl-query-result) ""))
-          (:publisher              ,(or (nth 11 spl-query-result) ""))
-          (:series                 ,(or (nth 12 spl-query-result) ""))
-          (:lang_code              ,(or (nth 13 spl-query-result) ""))
-          (:last_modified          ,(or (nth 14 spl-query-result) ""))))))
+  (if-let ((spl-query-result (if (listp query-result)
+                                 query-result
+                               (split-string (calibredb-chomp query-result) calibredb-sql-separator))))
+      `((:id                     ,(let ((id (nth 0 spl-query-result)))
+                                    (if (stringp id)
+                                        id
+                                      (number-to-string id))))
+        (:author-sort            ,(nth 1 spl-query-result))
+        (:book-dir               ,(nth 2 spl-query-result))
+        (:book-name              ,(nth 3 spl-query-result))
+        (:book-format  ,(downcase (or (nth 4 spl-query-result) "")))
+        (:book-pubdate           ,(nth 5 spl-query-result))
+        (:book-title             ,(nth 6 spl-query-result))
+        (:file-path    ,(concat (file-name-as-directory calibredb-root-dir)
+                                (file-name-as-directory (nth 2 spl-query-result))
+                                (nth 3 spl-query-result) "." (downcase (or (nth 4 spl-query-result) ""))))
+        (:tag                    ,(or (nth 7 spl-query-result) ""))
+        (:size                   ,(format "%.2f" (/ (let ((size (or (nth 8 spl-query-result) "")))
+                                                      (if (stringp size)
+                                                          (string-to-number size) size)) 1048576.0)))
+        (:comment                ,(or (nth 9 spl-query-result) ""))
+        (:ids                    ,(or (nth 10 spl-query-result) ""))
+        (:publisher              ,(or (nth 11 spl-query-result) ""))
+        (:series                 ,(or (nth 12 spl-query-result) ""))
+        (:lang_code              ,(or (nth 13 spl-query-result) ""))
+        (:last_modified          ,(or (nth 14 spl-query-result) "")))))
 
 (defun calibredb-getattr (my-alist key)
   "Get the attribute.
@@ -655,10 +678,11 @@ Argument PROPERTIES is for selecting different sql statement."
          (distinct (plist-get properties :distinct))
          (where (plist-get properties :where))
          (sql (format (cond
-                       (count "SELECT COUNT(id) FROM (SELECT * FROM (%s) %s)")
+                       (count calibredb-sql-count-string)
                        (distinct (concat "SELECT DISTINCT " distinct " FROM (SELECT * FROM (%s) %s)"))
                        (t "SELECT * FROM (%s) %s"))
                       (concat calibredb-query-string
+                              calibredb-sql-order-by-string
                               (pcase calibredb-sort-by
                                 ('id " id")
                                 ('title " title")
@@ -672,48 +696,43 @@ Argument PROPERTIES is for selecting different sql statement."
                                 (_ " id"))
                               (when (eq calibredb-order 'desc)
                                 " DESC"))
-                      (if where (concat " WHERE " where) "")))
-         (query-result (calibredb-query sql))
-         (line-list (if (and (functionp 'sqlite-available-p) (sqlite-available-p))
-                        query-result
-                      (split-string (calibredb-chomp query-result) calibredb-sql-newline) )))
-    ;; (message "%s" sql)
-    (cond (count (caar query-result))
-          (distinct query-result)
-          (t (cond ((equal "" query-result) '())
-                   ((equal nil query-result) '())
-                   ((numberp query-result) '())
-                   (t (let (res-list)
-                        (dolist (line line-list)
-                          (if (and (functionp 'sqlite-available-p) (sqlite-available-p))
-                              (push (calibredb-query-to-alist line) res-list)
-                            ;; validate if it is right format
-                            (if (string-match-p (concat "^[0-9]\\{1,10\\}" calibredb-sql-separator) line)
-                                ;; decode and push to res-list
-                                (push (calibredb-query-to-alist line) res-list))))
-                        (calibredb-getbooklist res-list))))))))
+                      (if where (concat " WHERE " where) ""))))
+    (when-let* ((query-result (calibredb-query sql))
+                (valid-query-result-p (or (listp query-result)
+                                          (if (stringp query-result)
+                                              (not (string= query-result "")))))
+                (line-list (if (listp query-result)
+                               query-result
+                             (split-string (calibredb-chomp query-result) calibredb-sql-newline) )))
+      ;; (message "%s" sql)
+      (cond (count (if (listp query-result)
+                       (caar line-list)
+                     (string-to-number (car line-list))))
+            (distinct line-list)
+            (t (let (res-list)
+                 (dolist (line line-list)
+                   (if (listp line)
+                       (push (calibredb-query-to-alist line) res-list)
+                     ;; validate if it is right format
+                     (if (string-match-p (concat "^[0-9]\\{1,10\\}" calibredb-sql-separator) line)
+                         ;; decode and push to res-list
+                         (push (calibredb-query-to-alist line) res-list))))
+                 (calibredb-getbooklist res-list)))))))
 
 (defun calibredb-candidate(id)
   "Generate one ebook candidate alist.
 ARGUMENT ID is the id of the ebook in string."
-  (let* ((query-result (calibredb-query (format "SELECT * FROM (%s) WHERE id = %s" calibredb-query-string id)))
-         (line-list (if (and (functionp 'sqlite-available-p) (sqlite-available-p))
-                        query-result
-                      (if query-result (split-string (calibredb-chomp query-result) calibredb-sql-newline)) )))
-    (cond ((equal "" query-result) '(""))
-          ((equal nil query-result) '(""))
-          (t (let (res-list)
-               (dolist (line line-list)
-                 (if (and (functionp 'sqlite-available-p) (sqlite-available-p))
-                     (push (calibredb-query-to-alist line) res-list)
-                   ;; validate if it is right format
-                   (if (string-match-p (concat "^[0-9]\\{1,10\\}" calibredb-sql-separator) line)
-                       ;; decode and push to res-list
-                       (push (calibredb-query-to-alist line) res-list)
-                     ;; concat the invalid format strings into last line
-                     ;; (setf (cadr (assoc :comment (car res-list))) (concat (cadr (assoc :comment (car res-list))) line))
-                     )))
-               (calibredb-getbooklist res-list)) ))))
+  (if-let* ((valid-id-p (stringp id))
+            (query-result (calibredb-query (format "SELECT * FROM (%s) WHERE id = %s" calibredb-query-string id)))
+            (valid-query-result-p (or (listp query-result)
+                                      (if (stringp query-result)
+                                          (not (string= query-result "")))))
+            (line (if (listp query-result) (car query-result)
+                    (car (split-string (calibredb-chomp query-result) calibredb-sql-newline))))
+            (valid-line-p (or (listp line)
+                              (string-match-p (concat "^[0-9]\\{1,10\\}" calibredb-sql-separator) line))))
+      (calibredb-getbooklist (list (calibredb-query-to-alist line) ))
+    '("")))
 
 (defun calibredb-format-item (book-alist)
   "Format the candidate string shown in helm or ivy.
@@ -741,12 +760,13 @@ Argument BOOK-ALIST ."
         (setq title (concat title "\n")))
     (format
      (if calibredb-detailed-view
-         (let ((num (cond (calibredb-format-all-the-icons 3)
+         (let ((num (cond (calibredb-format-nerd-icons 3)
+                          (calibredb-format-all-the-icons 3)
                           (calibredb-format-icons-in-terminal 3)
                           ((>= calibredb-id-width 0) calibredb-id-width)
                           (t 0 ))))
            (concat
-            "%s%s%s"
+            "%s %s%s"
             (calibredb-format-column (format "%sFormat:" (make-string num ? )) (+ 8 num) :left) "%s\n"
             (calibredb-format-column (format "%sDate:" (make-string num ? )) (+ 8 num) :left) "%s\n"
             (calibredb-format-column (format "%sAuthor:" (make-string num ? ))  (+ 8 num) :left) "%s\n"
@@ -754,18 +774,8 @@ Argument BOOK-ALIST ."
             (calibredb-format-column (format "%sIds:" (make-string num ? )) (+ 8 num) :left) "%s\n"
             (calibredb-format-column (format "%sComment:" (make-string num ? )) (+ 8 num) :left) "%s\n"
             (calibredb-format-column (format "%sSize:" (make-string num ? )) (+ 8 num) :left) "%s"))
-       "%s%s%s %s %s %s (%s) %s %s %s")
-     (cond (calibredb-format-all-the-icons
-            (concat (if (fboundp 'all-the-icons-icon-for-file)
-                        (all-the-icons-icon-for-file (calibredb-get-file-path (list book-alist))) "")
-                    " "))
-           (calibredb-format-icons-in-terminal
-            (concat (if (fboundp 'icons-in-terminal-icon-for-file)
-                        (icons-in-terminal-icon-for-file (calibredb-get-file-path (list book-alist) ) :v-adjust 0 :height 1) "")
-                    " "))
-           (calibredb-format-character-icons
-            (concat (calibredb-attach-icon-for (calibredb-get-file-path (list book-alist))) " "))
-           (t ""))
+       "%s %s%s %s %s %s (%s) %s %s %s")
+     (calibredb-icons-for-file-extensions format)
      (calibredb-format-column (format "%s" (propertize id 'face 'calibredb-id-face 'id id)) calibredb-id-width :left)
      (calibredb-format-column (format "%s%s"
                                       (if (s-contains? calibredb-favorite-keyword tag)
@@ -823,6 +833,40 @@ Argument BOOK-ALIST ."
              (if calibredb-size-show
                  (propertize "Mb" 'face 'calibredb-size-face) ""))) ))
 
+(defun calibredb-icons-for-file-extensions (extensions-string)
+  "Return the appropriate icon(s) for the given EXTENSIONS-STRING.
+If the string contains `calibredb-preferred-format`, only show that format.
+Otherwise, show icons for the first available format."
+  (let* ((extensions (split-string (string-trim extensions-string) ","))
+         (preferred-format (car (member calibredb-preferred-format extensions)))
+         (extensions-to-show (if preferred-format
+                                 (list preferred-format)
+                               (list (car extensions)))))
+    (cond
+     (calibredb-format-nerd-icons
+      (mapconcat (lambda (ext)
+                   (if (fboundp 'nerd-icons-icon-for-file)
+                       (nerd-icons-icon-for-file (concat "file." (string-trim ext)))
+                     ""))
+                 extensions-to-show " "))
+     (calibredb-format-all-the-icons
+      (mapconcat (lambda (ext)
+                   (if (fboundp 'all-the-icons-icon-for-file)
+                       (all-the-icons-icon-for-file (concat "file." (string-trim ext)))
+                     ""))
+                 extensions-to-show " "))
+     (calibredb-format-icons-in-terminal
+      (mapconcat (lambda (ext)
+                   (if (fboundp 'icons-in-terminal-icon-for-file)
+                       (icons-in-terminal-icon-for-file ext :v-adjust 0 :height 1)
+                     ""))
+                 extensions-to-show " "))
+     (calibredb-format-character-icons
+      (mapconcat (lambda (ext)
+                   (calibredb-attach-icon-for ext))
+                 extensions-to-show " "))
+     (t ""))))
+
 (provide 'calibredb-core)
 
 ;;; calibredb-core.el ends here
