@@ -1,7 +1,7 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "calibredb" "2.13.0"
+(define-package "calibredb" "20250913.1233"
   "Yet another calibre client."
-  '((emacs     "25.1")
+  '((emacs     "29.1")
     (org       "9.3")
     (transient "0.1.0")
     (s         "1.12.0")
@@ -9,8 +9,8 @@
     (request   "0.3.3")
     (esxml     "0.3.7"))
   :url "https://github.com/chenyanming/calibredb.el"
-  :commit "a12746cdd2605eba0ff88706473a04492dec84a2"
-  :revdesc "a12746cdd260"
+  :commit "99a234167a092bc0017d11c814f0b8c0da53a107"
+  :revdesc "99a234167a09"
   :keywords '("tools")
   :authors '(("Damon Chan" . "elecming@gmail.com"))
   :maintainers '(("Damon Chan" . "elecming@gmail.com")))
