@@ -46,6 +46,8 @@
 (declare-function calibredb-find-file-other-frame "calibredb-utils.el")
 (declare-function calibredb-open-file-with-default-tool "calibredb-utils.el")
 (declare-function calibredb-open-dired "calibredb-utils.el")
+(declare-function calibredb-infile-cover-path "calibredb-utils.el")
+(declare-function calibredb-extract-cover "calibredb-utils.el")
 (declare-function calibredb-catalog-bib-dispatch "calibredb-transient.el")
 (declare-function calibredb-export-dispatch "calibredb-transient.el")
 (declare-function calibredb-edit-annotation "calibredb-annotation.el")
@@ -66,6 +68,8 @@
 (declare-function calibredb-virtual-library-list "calibredb-library.el")
 (declare-function calibredb-virtual-library-next "calibredb-library.el")
 (declare-function calibredb-virtual-library-previous "calibredb-library.el")
+(declare-function calibredb-folder-candidates "calibredb-folder.el")
+(declare-function calibredb-folder-update-tags-by-lpath "calibredb-folder.el")
 
 (defcustom calibredb-search-filter ""
   "Query string filtering shown entries."
@@ -260,7 +264,7 @@ Optional argument SWITCH to switch to *calibredb-search* buffer to other window.
                                                     (propertize ext
                                                                 'face 'calibredb-format-face
                                                                 'mouse-face 'calibredb-mouse-face
-                                                                'help-echo (if (s-contains? "http" file)
+                                                                'help-echo (if (string-prefix-p "http" file)
                                                                                file
                                                                              (expand-file-name
                                                                               (concat (file-name-base file) "." ext)
@@ -331,7 +335,13 @@ Optional argument SWITCH to switch to *calibredb-search* buffer to other window.
 
 (defun calibredb-search-buffer ()
   "Create buffer calibredb-search."
-  (get-buffer-create "*calibredb-search*"))
+  (let ((buffer (get-buffer "*calibredb-search*")))
+    (if (buffer-live-p buffer)
+        buffer
+      (setq buffer (get-buffer-create "*calibredb-search*"))
+      (with-current-buffer buffer
+        (calibredb-search-mode)))
+    buffer))
 
 (defun calibredb-search-header ()
   "TODO: Return the string to be used as the Calibredb header.
@@ -403,6 +413,7 @@ Indicating the library you use."
       (add-to-list 'ivy-alt-done-functions-alist '(calibredb-add . ivy--directory-done)))
   (add-hook 'minibuffer-setup-hook #'calibredb-search--minibuffer-setup)
   (add-to-list 'mailcap-mime-extensions '(".epub" . "application/epub+zip"))
+  (add-to-list 'mailcap-mime-extensions '(".azw3" . "application/x-mobi8-ebook"))
   (add-to-list 'mailcap-mime-extensions '(".mobi" . "application/x-mobipocket-ebook")))
 
 (defun calibredb-search-mouse (event)
@@ -437,7 +448,7 @@ Argument EVENT mouse event."
     (if (not (equal calibredb-search-filter ""))
         (progn
           (calibredb-search-refresh)
-          (calibredb-search-update-buffer))
+          (calibredb-search-update-buffer-by-library-type))
       (calibredb-search-refresh))
     (set-window-start (selected-window) pos)
     (goto-char beg)
@@ -450,7 +461,7 @@ Argument EVENT mouse event."
     (if (not (equal calibredb-search-filter ""))
         (progn
           (calibredb-search-refresh)
-          (calibredb-search-update-buffer))
+          (calibredb-search-update-buffer-by-library-type))
       (calibredb-search-refresh))
     (while (not (equal id (calibredb-read-metadatas "id")))
       (forward-line 1))
@@ -535,7 +546,9 @@ Argument EVENT mouse event."
 
 (defun calibredb-condense-comments (str)
   "Condense whitespace in STR into a single space."
-  (replace-regexp-in-string "[[:space:]\n\r]+" " " str))
+  (replace-regexp-in-string
+   "[[:space:]\n\r]+" " "
+   (replace-regexp-in-string "<[^>]+>" "" str)))
 
 (defun calibredb-favorite-mouse-1 (event)
   "Visit the location click on.
@@ -680,34 +693,54 @@ Argument KEYWORD is the tag keyword."
 Argument CANDS is the list of candiates.
 Argument KEYWORD is the metadata keyword to be toggled."
   (let* ((cand (pop cands))
-         (tags (calibredb-read-metadatas "tags" cand)))
-    ;; (pp cand)
-    (if cand
-        (set-process-sentinel
-         (let* ((id (calibredb-getattr cand :id)))
-           (if (s-contains? keyword tags)
+         (tags (calibredb-read-metadatas "tags" cand))
+         (lpath (calibredb-getattr cand :lpath)))
+    (if lpath
+        (progn
+          (if (s-contains? keyword tags)
+              (calibredb-folder-update-tags-by-lpath lpath (s-replace keyword "" tags))
+            (calibredb-folder-update-tags-by-lpath lpath keyword))
+          (cond ((equal major-mode 'calibredb-show-mode)
+                 (calibredb-show-refresh))
+                ((eq major-mode 'calibredb-search-mode)
+                 (calibredb-search-refresh-or-resume))
+                (t nil)))
+      ;; (pp cand)
+      (if cand
+          (set-process-sentinel
+           (let* ((id (calibredb-getattr cand :id)))
+             (if (s-contains? keyword tags)
+                 (calibredb-process :command "set_metadata"
+                                    :option (format "--field tags:\"%s\"" (s-replace keyword "" tags))
+                                    :id id
+                                    :library (format "--library-path \"%s\"" calibredb-root-dir))
                (calibredb-process :command "set_metadata"
-                                  :option (format "--field tags:\"%s\"" (s-replace keyword "" tags))
+                                  :option (format "--field tags:\"%s,%s\"" tags keyword)
                                   :id id
-                                  :library (format "--library-path \"%s\"" calibredb-root-dir))
-             (calibredb-process :command "set_metadata"
-                                :option (format "--field tags:\"%s,%s\"" tags keyword)
-                                :id id
-                                :library (format "--library-path \"%s\"" calibredb-root-dir))))
-         (lambda (p _e)
-           (when (= 0 (process-exit-status p))
-             (calibredb-toggle-metadata-process cands keyword))))
-      ;; if no candidate left to be processed, refresh *calibredb-search*
-      (cond ((equal major-mode 'calibredb-show-mode)
-             (calibredb-show-refresh))
-            ((eq major-mode 'calibredb-search-mode)
-             (calibredb-search-refresh-or-resume))
-            (t nil)))))
+                                  :library (format "--library-path \"%s\"" calibredb-root-dir))))
+           (lambda (p _e)
+             (when (= 0 (process-exit-status p))
+               (calibredb-toggle-metadata-process cands keyword))))
+        ;; if no candidate left to be processed, refresh *calibredb-search*
+        (cond ((equal major-mode 'calibredb-show-mode)
+               (calibredb-show-refresh))
+              ((eq major-mode 'calibredb-search-mode)
+               (calibredb-search-refresh-or-resume))
+              (t nil))))))
+
 ;; live filtering
 
-(defun calibredb-search-get-filterred-entries (&optional page)
-  "Get ebook candidate entries by PAGE."
-  (calibredb-search-candidates calibredb-search-filter :limit calibredb-search-page-max-rows :page page))
+(defun calibredb-search-get-filterred-entries (&rest properties)
+  "Get ebook candidate entries by DB, OPDS or FOLDER."
+  (let ((db (plist-get properties :db))
+        (page (plist-get properties :page))
+        (opds (plist-get properties :opds))
+        (folder (plist-get properties :folder)))
+    (cond
+     (db
+      (calibredb-search-candidates calibredb-search-filter :limit (calibredb-search-page-max-rows) :page page))
+     (opds opds)
+     (folder (calibredb-folder-candidates calibredb-search-filter)))))
 
 (defun calibredb-search-print-entry--default (entry)
   "Print ENTRY to the buffer."
@@ -735,7 +768,7 @@ Argument KEYWORD is the metadata keyword to be toggled."
         (with-current-buffer buffer
           (let ((calibredb-search-filter current-filter))
             (setq calibredb-search-current-page 1)
-            (calibredb-search-update-buffer)))))))
+            (calibredb-search-update-buffer-by-library-type)))))))
 
 (defun calibredb-search-live-filter ()
   "Filter the calibredb-search buffer as the filter is written.
@@ -772,36 +805,65 @@ ebook record will be shown.
                                              (calibredb-format-filter-p "(format)")
                                              (t "(live)"))) calibredb-search-filter))
         (message calibredb-search-filter))
-    (calibredb-search-update-buffer)))
+    (calibredb-search-update-buffer-by-library-type)))
+
+(defun calibredb-search-update-buffer-by-library-type ()
+  "Update the calibredb-search buffer by library type, opds, metadata or
+folder meatadata."
+  (cond
+   ((string-prefix-p "http" calibredb-root-dir)
+    (message "OPDS does not suppprt search at this moment."))
+   ((and (stringp calibredb-db-dir)
+         (file-exists-p calibredb-db-dir)
+         (s-contains? "metadata.db" calibredb-db-dir))
+    (calibredb-search-update-buffer))
+   ((and (file-exists-p (expand-file-name ".metadata.calibre" calibredb-root-dir)))
+    (calibredb-search-update-buffer :folder t))))
 
 (defun calibredb-search-keyword-filter (keyword)
   "Filter the calibredb-search buffer with KEYWORD."
   (setq calibredb-search-filter keyword)
-  (calibredb-search-update-buffer))
+  (calibredb-search-update-buffer-by-library-type))
 
-(defun calibredb-search-update-buffer (&optional page)
+(defvar calibredb-search-entries-length 0
+  "The number of entries in the current search result.")
+
+(defun calibredb-search-update-buffer (&rest properties)
   "Update the calibredb-search buffer listing to match the database with PAGE."
   (interactive)
   (with-current-buffer (calibredb-search-buffer)
-    (let* ((inhibit-read-only t)
+    (let* ((opds (plist-get properties :opds))
+           (folder (plist-get properties :folder))
+           (page (or (plist-get properties :page) 1))
+           (db (if (or opds folder) nil t))
+           (inhibit-read-only t)
            (standard-output (current-buffer))
            (id 0)
-           (entries (calibredb-search-get-filterred-entries page))
-           (len (length entries)))
-      (setq calibredb-search-entries-length (calibredb-search-candidates calibredb-search-filter :count t))
-      (setq calibredb-search-pages (ceiling calibredb-search-entries-length calibredb-search-page-max-rows))
+           (entries (calibredb-search-get-filterred-entries :db db :page page :opds opds :folder folder))
+           (len (length entries))
+           (rows (calibredb-search-page-max-rows)))
+      (setq calibredb-search-entries-length (if db (calibredb-search-candidates calibredb-search-filter :count t) len))
+      (setq calibredb-search-pages (if db (ceiling calibredb-search-entries-length rows) 1))
       (erase-buffer)
       ;; reset calibredb-virtual-library-name
       (unless (-contains? (mapcar #'cdr calibredb-virtual-library-alist) calibredb-search-filter)
         (setq calibredb-virtual-library-name calibredb-virtual-library-default-name))
       (dolist (entry entries)
         (setq id (1+ id))
-        (when (<= id calibredb-search-page-max-rows)
+        (when (<= id (if db rows len))
           (funcall calibredb-search-print-entry-function entry)
           (insert "\n")))
       (if (< len calibredb-search-entries-length)
           (dotimes (i calibredb-search-pages)
-            (insert " " (buttonize (format "%d" (1+ i)) #'calibredb-search-more-data (1+ i)) " "))
+            (let ((button-string (format "%d" (1+ i))))
+              (if (equal (string-to-number button-string) calibredb-search-current-page)
+                  (add-face-text-property 0 (length button-string) 'calibredb-current-page-button-face t button-string))
+              (let ((col (current-column))
+                    (win-width (window-width))
+                    (button (buttonize button-string #'calibredb-search-more-data (1+ i))))
+                (when (> (+ col (length button)) win-width)
+                  (insert "\n"))
+                (insert " " button " "))))
         (insert "End of entries.\n"))
       (goto-char (point-min))         ; back to point-min after filtering
       (setf calibredb-search-last-update (float-time))
@@ -813,7 +875,7 @@ ebook record will be shown.
     (setq calibredb-search-current-page page)
     (beginning-of-line)
     (delete-region (point) (progn (forward-line 1) (point)))
-    (calibredb-search-update-buffer page)))
+    (calibredb-search-update-buffer :page page)))
 
 
 (defun calibredb-search-next-page ()
@@ -822,7 +884,7 @@ ebook record will be shown.
   (if (< calibredb-search-current-page calibredb-search-pages)
       (progn
         (setq calibredb-search-current-page (1+ calibredb-search-current-page))
-        (calibredb-search-update-buffer calibredb-search-current-page) )
+        (calibredb-search-update-buffer :page calibredb-search-current-page))
     (message "Last page.")))
 
 (defun calibredb-search-previous-page ()
@@ -831,7 +893,7 @@ ebook record will be shown.
   (if (> calibredb-search-current-page 1)
       (progn
         (setq calibredb-search-current-page (1- calibredb-search-current-page))
-        (calibredb-search-update-buffer calibredb-search-current-page) )
+        (calibredb-search-update-buffer :page calibredb-search-current-page) )
     (message "First page.")))
 
 
@@ -851,6 +913,17 @@ ebook record will be shown.
       (string-match-p regexp ""))))
 
 
+(defcustom calibredb-search-page-max-rows-auto-adjust nil
+  "WORKAROUND: When non-nil, auto adjust the max rows of the page.
+It is unstable when live search, and the algorithm is also inaccurate."
+  :group 'calibredb
+  :type 'boolean)
+
+(defcustom calibredb-search-page-max-rows-auto-adjust-offset 4
+  "WORKAROUND: The offset when auto adjust the max rows.
+It may not be accurate, but it is a good guess."
+  :group 'calibredb
+  :type 'integer)
 
 (defcustom calibredb-search-page-max-rows 44
   "The maximum number of entries to display in a single page."
@@ -863,11 +936,41 @@ ebook record will be shown.
 (defvar calibredb-search-pages 0
   "The number of pages in the current search result.")
 
+(defvar calibredb-search-buffer-line-pixel-height nil
+  "The pixel height of the *calibredb-search* buffer line.")
+
+(defun calibredb-search-page-max-rows ()
+  "Return the maximum number of entries to display.
+In the *calibredb-search* window."
+  (let ((win (get-buffer-window "*calibredb-search*" 'visible)))
+    (if calibredb-search-page-max-rows-auto-adjust
+        (if (window-live-p win)
+            (progn
+              (unless calibredb-search-buffer-line-pixel-height
+                (setq calibredb-search-buffer-line-pixel-height (line-pixel-height)))
+              (let* ((window-pixel-height (window-pixel-height win))
+                     (font-height calibredb-search-buffer-line-pixel-height)
+                     (offset (* calibredb-search-page-max-rows-auto-adjust-offset font-height)))  ;; Height of mode line
+              ;; Calculate visible height by subtracting header and mode line heights
+              (let ((visible-pixel-height (- window-pixel-height offset)))
+                ;; Calculate the number of lines that fit in the visible height
+                (max 1 (floor visible-pixel-height font-height)))) )
+          calibredb-search-page-max-rows)
+      calibredb-search-page-max-rows)))
+
+
+(defun calibredb-sanitize-filter (filter)
+  "Sanitize FILTER for use in SQL queries by escaping special characters."
+  (let ((sanitized filter))
+    ;; Escape single quotes by doubling them
+    (setq sanitized (replace-regexp-in-string "'" "''" sanitized))
+    sanitized))
+
 (defun calibredb-search-candidates (filter &rest properties)
   "Generate ebook candidate alist.
 Argument: FILTER is the filter string.
 Argument: PROPERTIES is the addiontal parameters."
-  (let* ((words (split-string filter " "))
+  (let* ((words (split-string (calibredb-sanitize-filter filter) " "))
          (limit (plist-get properties :limit))
          (count (plist-get properties :count))
          (page (plist-get properties :page)))
@@ -941,7 +1044,7 @@ Argument: PROPERTIES is the addiontal parameters."
              (when limit
                (format " LIMIT %s " limit) )
              (when page
-               (format " OFFSET %s " (* (1- page) calibredb-search-page-max-rows)))
+               (format " OFFSET %s " (* (1- page) (calibredb-search-page-max-rows))))
 
              )
      :count count)))
@@ -955,36 +1058,38 @@ Argument: PROPERTIES is the addiontal parameters."
   (setq calibredb-detailed-view (not calibredb-detailed-view))
   (calibredb-search-toggle-view-refresh))
 
-(defun calibredb-detail-view-insert-image (entry)
-  "Insert image in *calibredb-search* under detail view based on ENTRY."
-  (if (and calibredb-detial-view calibredb-detailed-view-image-show)
-      (let ((num (cond (calibredb-format-all-the-icons 3)
-                       (calibredb-format-icons-in-terminal 3)
-                       ((>= calibredb-id-width 0) calibredb-id-width)
-                       (t 0 ))))
-        (insert "\n")
-        (insert (make-string num ? ))
-        (calibredb-insert-image (calibredb-get-cover (cdr entry)) "" calibredb-detailed-view-image-max-width calibredb-detailed-view-image-max-height))))
-
 (defun calibredb-detailed-view-insert-image (entry)
   "Insert image in *calibredb-search* under detailed view based on ENTRY."
   (if (and calibredb-detailed-view calibredb-detailed-view-image-show)
-      (let* ((num (cond (calibredb-format-all-the-icons 3)
+      (let* ((num (cond (calibredb-format-nerd-icons 3)
+                        (calibredb-format-all-the-icons 3)
                         (calibredb-format-icons-in-terminal 3)
                         ((>= calibredb-id-width 0) calibredb-id-width)
                         (t 0 )))
              (file (calibredb-getattr (cdr entry) :file-path))
              (format (calibredb-getattr (cdr entry) :book-format))
-             (cover (concat (file-name-directory file) "cover.jpg")))
-          (if (image-type-available-p (intern format))
-              (progn
-                (insert "\n")
-                (insert (make-string num ? ))
-                (calibredb-insert-image file "" calibredb-detailed-view-image-max-width calibredb-detailed-view-image-max-height))
-            (progn
-              (insert "\n")
-              (insert (make-string num ? ))
-              (calibredb-insert-image cover "" calibredb-detailed-view-image-max-width calibredb-detailed-view-image-max-height))))))
+             (cover (concat (file-name-directory file) "cover.jpg"))
+             (infile-cover (calibredb-infile-cover-path file)))
+        (cond ((image-type-available-p (intern format))
+               (progn
+                 (insert "\n")
+                 (insert (make-string num ? ))
+                 (calibredb-insert-image file "" calibredb-detailed-view-image-max-width calibredb-detailed-view-image-max-height)))
+              ((file-exists-p cover)
+               (insert "\n")
+               (insert (make-string num ? ))
+               (calibredb-insert-image cover "" calibredb-detailed-view-image-max-width calibredb-detailed-view-image-max-height))
+              ((file-exists-p infile-cover)
+               (insert "\n")
+               (insert (make-string num ? ))
+               (calibredb-insert-image infile-cover "" calibredb-detailed-view-image-max-width calibredb-detailed-view-image-max-height))
+              (t
+               ;; extract cover
+               (calibredb-extract-cover (cdr entry))
+               (when (file-exists-p infile-cover)
+                 (insert "\n")
+                 (insert (make-string num ? ))
+                 (calibredb-insert-image infile-cover "" calibredb-detailed-view-image-max-width calibredb-detailed-view-image-max-height)))))))
 
 (defun calibredb-toggle-view-at-point ()
   "Toggle between detailed view or compact view in *calibredb-search* buffer at point."
@@ -1114,19 +1219,24 @@ Argument: PROPERTIES is the addiontal parameters."
     (kill-new
      (with-temp-buffer
        (dolist (cand candidates)
-         (let ((id (calibredb-getattr cand :id))
-               (path (calibredb-get-file-path cand t))
-               (title (calibredb-getattr cand :book-title)))
-           (insert (format "[[file:%s][%s %s - %s]]\n"
-                           path
-                           (cond (calibredb-format-all-the-icons
-                                  (if (fboundp 'all-the-icons-icon-for-file)
-                                      (all-the-icons-icon-for-file path) ""))
-                                 (calibredb-format-icons-in-terminal
-                                  (if (fboundp 'icons-in-terminal-icon-for-file)
-                                      (icons-in-terminal-icon-for-file path :v-adjust 0 :height 1) ""))
-                                 (t "")) id title))
-           (message "Copied: %s - \"%s\" as org link." id title)))
+         (let* ((id (calibredb-getattr cand :id))
+                (author (calibredb-getattr cand :author-sort))
+                (path (calibredb-get-file-path cand t))
+                (title (calibredb-getattr cand :book-title))
+                (link (format "[[file:%s][%s%s - %s]]\n"
+                              path
+                              (cond (calibredb-format-all-the-icons
+                                     (if (fboundp 'nerd-icons-icon-for-file)
+                                         (nerd-icons-icon-for-file path) ""))
+                                    (calibredb-format-all-the-icons
+                                     (if (fboundp 'all-the-icons-icon-for-file)
+                                         (all-the-icons-icon-for-file path) ""))
+                                    (calibredb-format-icons-in-terminal
+                                     (if (fboundp 'icons-in-terminal-icon-for-file)
+                                         (icons-in-terminal-icon-for-file path :v-adjust 0 :height 1) ""))
+                                    (t "")) title author)))
+           (insert link)
+           (message "Copied (org file link): %s" link)))
        (buffer-string)))
     ;; remove overlays and text properties
     (let* ((beg (point-min))
