@@ -74,7 +74,7 @@
   ["Other operation"
    [("b" "BibTex"   calibredb-catalog-bib-dispatch)
     ("i" "Edit Annotation" calibredb-edit-annotation)]
-   [("'" "Search with rga" calibredb-rga)
+   [("'" "Search with rga" calibredb-consult-ripgrep-all)
     ("/" "Live Filter" calibredb-search-live-filter)]
    [("y" "Yank" calibredb-yank-dispatch)]]
   [("q" "Quit"   transient-quit-one)])
@@ -91,7 +91,7 @@
    [("e" "Export" calibredb-export-dispatch)
     ("s" "set_metadata"   calibredb-set-metadata-dispatch)
     ("y" "Yank"   calibredb-yank-dispatch)
-    ("'" "Search with rga" calibredb-rga)]]
+    ("'" "Search with rga" calibredb-consult-ripgrep-all)]]
   [("q" "Quit"   transient-quit-one)])
 
 (transient-define-prefix calibredb-set-metadata-dispatch ()
@@ -260,7 +260,11 @@
   :man-page "calibredb"
   ["Yank operaion"
    [("y" "Copy as calibredb org links"         calibredb-org-link-copy)
-    ("f" "Copy as file org links"              calibredb-copy-as-org-link)]]
+    ("f" "Copy as file org links"              calibredb-copy-as-org-link)
+    ("t" "Copy titles"                         calibredb-org-title-copy)
+    ("m" "Copy as org-protocol markdown links" calibredb-org-protocol-link-markdown-copy)
+    ("M" "Copy as markdown links"              calibredb-org-markdown-copy)
+    ("o" "Copy as org-protocol links"          calibredb-org-protocol-link-copy)]]
   [("q" "Quit"   transient-quit-one)])
 
 (defun calibredb-transient-read-bib-fields (prompt _initial-input _history)
