@@ -109,6 +109,16 @@ Argument FILEPATH is the file path."
                         "open" (expand-file-name filepath)))
         (t (message "unknown system!?"))))
 
+(defun calibredb-list-ebooks-files (directory file-extensions)
+  "Recursively list all ebook files with FILE-EXTENSIONS in DIRECTORY.
+FILE-EXTENSIONS should be a list of strings."
+  (let* ((regex (format "\\.%s\\'" (regexp-opt file-extensions)))
+         (files (directory-files-recursively directory regex t)))
+    (cond ((> (length files) 0)
+           files)
+          (t (error "No Ebooks found in %s" directory)
+             nil))))
+
 (defun calibredb-get-file-path (entry &optional prompt)
   "Get file path from a valid candidate ENTRY.
 Optional argument PROMPT to Select a format."
@@ -116,16 +126,21 @@ Optional argument PROMPT to Select a format."
     (cond ((s-equals? "" file-path) "")         ; no file-path field
           ((file-exists-p file-path) file-path) ; default file-path is a valid file
           ((calibredb-local-file-exists-p entry) (calibredb-local-file entry)) ; valid local file is found
-          ((s-contains? "http" file-path) file-path) ; for http link, just return
+          ((string-prefix-p "http" file-path) file-path) ; for http link, just return
           (t (if (s-contains? "," (file-name-extension file-path)) ; try to split the extension (for example, it may be epub,pdf) and return the first format
                  (let* ((parent (file-name-directory file-path))
                         (filename (file-name-base file-path))
                         (ext (s-split "," (file-name-extension file-path)))
-                        (files (-map (lambda (e) (expand-file-name (concat filename "." e) parent)) ext)))
+                        (files (calibredb-list-ebooks-files parent ext)))
                    (if calibredb-preferred-format
-                       (or (-first (lambda (f) (string= (file-name-extension f) calibredb-preferred-format)) files) (car files))
+                       (let ((preferred-files (-filter (lambda (f) (string= (file-name-extension f) calibredb-preferred-format)) files)))
+                         (cond ((> (length preferred-files) 1)
+                                (completing-read (format "Select a file (%s): " calibredb-preferred-format) preferred-files))
+                               ((= (length preferred-files) 1)
+                                (car preferred-files))
+                               (t (car files))))
                      (if prompt
-                         (completing-read "Select a format: " files)
+                         (completing-read (format "Select a file %s: " ext) files)
                        (car files))))
                file-path)))))           ; if extension does not have comma, at last just retrun it.
 
@@ -146,12 +161,19 @@ Download it if book-cover is non-nil."
   (let ((file-path (calibredb-getattr entry :file-path))
         (book-format (calibredb-getattr entry :book-format))
         (book-cover (calibredb-getattr entry :book-cover)))
-    (pp book-cover)
+    ;; (pp book-cover)
     (cond ((image-type-available-p (intern book-format)) file-path) ; the file is an image
           ((file-exists-p (concat (file-name-directory file-path) "cover.jpg"))
            (concat (file-name-directory file-path) "cover.jpg")) ; cover.jpg exists
+          ((file-exists-p (calibredb-infile-cover-path file-path))
+           (calibredb-infile-cover-path file-path))
           ((not book-cover)                                      ; book-cover is nil, use default cover
-           (expand-file-name "cover.jpg" calibredb-images-path))
+           ;; first try to extract cover from the file
+           (let ((infile-cover (calibredb-infile-cover-path file-path)))
+             (calibredb-extract-cover entry)
+             (if (file-exists-p infile-cover)
+                 infile-cover
+               (expand-file-name "cover.jpg" calibredb-images-path))))
           ((s-contains? "base64" book-cover)
            (if (string-match "data:image/\\(.*\\);base64,\\(.*\\)" book-cover)
                (let ((cover (expand-file-name (format "cover.%s" (match-string 1 book-cover)) temporary-file-directory)))
@@ -183,6 +205,33 @@ Download it if book-cover is non-nil."
              filename))
           (t (expand-file-name "cover.jpg" calibredb-images-path))))) ;return the default image
 
+(defun calibredb-extract-cover (entry)
+  "Extract ENTRY and save the cover to the same directory."
+  ;; only extract cover if calibredb-root-dir is not a http link
+  (unless (string-prefix-p "http" calibredb-root-dir)
+    ;; only extract cover if ebook-meta is available
+    (when (executable-find calibredb-ebook-meta-program)
+      ;; extract cover
+      (let* ((file (calibredb-getattr entry :file-path))
+             (cover (calibredb-infile-cover-path file)))
+        (call-process-shell-command (format "%s %s --get-cover %s" calibredb-ebook-meta-program (shell-quote-argument file) (shell-quote-argument cover)) nil "*ebook-meta*")
+        (if (file-exists-p cover)
+            (message "Saved %s" cover)
+          ;; can not generate cover.jpg, try to copy cover.jpg from calibredb-images-path
+          (copy-file (expand-file-name "cover.jpg" calibredb-images-path)
+                     cover))))))
+
+(defun calibredb-infile-cover-path (file)
+  "Extract FILE and return the cover path."
+  (let* ((ori-file-directory (file-name-directory file))
+         (file-directory (concat ori-file-directory "covers"))
+         (file-base (file-name-base file))
+         (cover (expand-file-name (concat file-base ".jpg") file-directory)))
+    (unless (file-exists-p file-directory)
+      (make-directory file-directory))
+    cover))
+
+
 (defun calibredb-insert-image (path alt width height)
   "Insert an image for PATH at point with max WIDTH and max HEIGTH, falling back to ALT."
   (cond
@@ -212,20 +261,20 @@ Optional argument CANDIDATE is the selected item."
                   (let ((calibredb-preferred-format nil))
                     (calibredb-get-file-path candidate t))
                 (calibredb-get-file-path candidate t))))
-    (cond ((s-contains? "http" file)
+    (cond ((string-prefix-p "http" file)
            (let ((url (calibredb-getattr candidate :file-path))
                  (title (calibredb-getattr candidate :book-title))
                  (type (calibredb-getattr candidate :book-format)))
              (if (s-equals-p title "search") ; TODO: Workaround, now it only works with calibre-web
-                 (calibredb-opds-search calibredb-root-dir)
+                 (calibredb-opds-search url)
                (message url)
                (message type)
                (let ((library (-first (lambda (lib)
                                         (s-contains? (file-name-directory (car lib)) url))
                                       calibredb-library-alist)))
-                 (if (calibredb-opds-mailcap-mime-to-extn type)
-                     (calibredb-opds-download title url (calibredb-opds-mailcap-mime-to-extn type) (nth 1 library) (nth 2 library))
-                   (calibredb-opds-request-page url (nth 1 library) (nth 2 library)))))) )
+                 (if (string-match (regexp-opt '("atom" "xml")) type)
+                     (calibredb-opds-request-page url (assoc-default 'account library) (assoc-default 'password library))
+                   (calibredb-opds-download title url type (assoc-default 'account library) (assoc-default 'password library)))))))
           ((s-equals? "" file) (message "No files."))
           (t (find-file file)))))
 
@@ -471,25 +520,36 @@ Argument PROPS are the additional parameters."
 Argument CANDS is the list of candiates.
 Argument FIELD is the metadata field, e.g. tags, author.
 Argument INPUT is the metadata contents to be set."
-  (let ((cand (pop cands)))
-    ;; (pp cand)
-    (if cand
-        (set-process-sentinel
-         (let* ((id (calibredb-getattr cand :id)))
-           (calibredb-process :command "set_metadata"
-                              :option "--field"
-                              :input (format "%s:\"%s\"" field input)
-                              :id id
-                              :library (format "--library-path \"%s\"" calibredb-root-dir)))
-         (lambda (p _e)
-           (when (= 0 (process-exit-status p))
-             (calibredb-set-metadata-process cands field input))))
-      ;; if no candidate left to be processed, refresh *calibredb-search*
-      (cond ((equal major-mode 'calibredb-show-mode)
-             (calibredb-show-refresh))
-            ((eq major-mode 'calibredb-search-mode)
-             (calibredb-search-refresh-or-resume))
-            (t nil)))))
+  (let* ((cand (pop cands))
+         (lpath (calibredb-getattr cand :lpath)))
+    (if lpath
+        (pcase field
+          ("tags"
+           (calibredb-folder-update-tags-by-lpath lpath input)
+           (cond ((equal major-mode 'calibredb-show-mode)
+                  (calibredb-show-refresh))
+                 ((eq major-mode 'calibredb-search-mode)
+                  (calibredb-search-refresh-or-resume))
+                 (t nil)))
+          (_ (message "We do not support setting %s for .matadata.calibre at this moment." calibredb-db-dir)))
+      ;; (pp cand)
+      (if cand
+          (set-process-sentinel
+           (let* ((id (calibredb-getattr cand :id)))
+             (calibredb-process :command "set_metadata"
+                                :option "--field"
+                                :input (format "%s:\"%s\"" field input)
+                                :id id
+                                :library (format "--library-path \"%s\"" calibredb-root-dir)))
+           (lambda (p _e)
+             (when (= 0 (process-exit-status p))
+               (calibredb-set-metadata-process cands field input))))
+        ;; if no candidate left to be processed, refresh *calibredb-search*
+        (cond ((equal major-mode 'calibredb-show-mode)
+               (calibredb-show-refresh))
+              ((eq major-mode 'calibredb-search-mode)
+               (calibredb-search-refresh-or-resume))
+              (t nil))))))
 
 
 (defun calibredb-set-metadata--tags (&optional candidate)
@@ -1053,7 +1113,9 @@ Argument FIELD table name in database."
      (seq-uniq
       (let (l)
         (cl-loop for item in (calibredb-candidates :distinct ,(format "%s" field )) do
-                 (if (car item) (setq l (append (split-string (car item ) ",") l)) "" )) l))))
+                 (if (listp item)
+                     (if (car item) (setq l (append (split-string (car item ) ",") l)) "" )
+                   (setq l (append (split-string item ",") l)))) l))))
 
 (calibredb-all "id")
 (calibredb-all "author_sort")
